/* $Id$ -*- C++ -*-
 */

#include "contour.h"
#include <cstring>

// Fortran types
typedef float real;
typedef int integer;

extern "C" {
  void tvfmfs_metric_(real* erp, real* haat, integer* channel, real* field,
		      real* distance, integer* iswitch, integer* curve, char* flag);
}

void fix_contours(Station* s)
{
  integer channel = s->_channel;

  for (Station::iterator i = s->begin(); i != s->end(); ++i) {
    real erp = i->_power;
    real haat = i->_haat;
    real distance = NA;
    integer iswitch = 2; // compute distance from field
    char flag[] = "      ";
    
    integer curve = 0;   // F(50,50)
    real field = 60;
    tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
    i->_p60 = distance;
    
    curve = 1;   // F(50,10)
    field = 40;
    tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
    i->_i40 = distance;

    curve = 1;   // F(50,10)
    field = 54;
    tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
    i->_i54 = distance;
    
    curve = 1;   // F(50,10)
    field = 100;
    tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
    i->_i100 = distance;
  }
}

int main(int argc, char* argv[])
{
  Station* s = NULL;
  int arg = 1;
  if (argv[1][0] != '-') {
    std::ifstream f(argv[1]);
    s = new Station(f);
    ++arg;
  }else{
    s = new Station(std::cin);
  }

  for (int i = arg; i < argc; ++i) {
    if (strcmp(argv[i], "-field") == 0) {
      for (Station::iterator i = s->begin(); i != s->end(); ++i) {
	i->_antenna_rel_db = 20 * log10(i->_antenna_rel_field);
	i->_power = s->_power * pow(10, i->_antenna_rel_db/10);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-db") == 0) {
      for (Station::iterator i = s->begin(); i != s->end(); ++i) {
	i->_antenna_rel_field = pow(10, i->_antenna_rel_db/20);
	i->_power = s->_power * pow(10, i->_antenna_rel_db/10);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-power") == 0) {
      for (Station::iterator i = s->begin(); i != s->end(); ++i) {
	i->_antenna_rel_db = 10 * log10(i->_power / s->_power);
	i->_antenna_rel_field = pow(10, i->_antenna_rel_db/20);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-p60") == 0) {
      integer channel = s->_channel;
      for (Station::iterator i = s->begin(); i != s->end(); ++i) {
	real erp = i->_power;
	real haat = i->_haat;
	real distance = i->_p60;
	integer iswitch = 1; // compute field from distance;
	char flag[] = "      ";
	integer curve = 0;   // F(50,50)
	real field = NA;
	for (int ii=0; ii<2; ++ii) {
	  tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
	  double adjustment = 60 - field;
	  i->_antenna_rel_db += adjustment;
	  erp = i->_power = s->_power * pow(10, i->_antenna_rel_db/10);
	}
	i->_antenna_rel_field = pow(10, i->_antenna_rel_db/20);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-i40") == 0) {
      integer channel = s->_channel;
      for (Station::iterator i = s->begin(); i != s->end(); ++i) {
	real erp = i->_power;
	real haat = i->_haat;
	real distance = i->_i40;
	integer iswitch = 1; // compute field from distance;
	char flag[] = "      ";
	integer curve = 1;   // F(50,10)
	real field = NA;
	for (int ii=0; ii<2; ++ii) {
	  tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
	  double adjustment = 40 - field;
	  i->_antenna_rel_db += adjustment;
	  erp = i->_power = s->_power * pow(10, i->_antenna_rel_db/10);
	}
	i->_antenna_rel_field = pow(10, i->_antenna_rel_db/20);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-i54") == 0) {
      integer channel = s->_channel;
      for (Station::iterator i = s->begin(); i != s->end(); ++i) {
	real erp = i->_power;
	real haat = i->_haat;
	real distance = i->_i54;
	integer iswitch = 1; // compute field from distance;
	char flag[] = "      ";
	integer curve = 1;   // F(50,10)
	real field = NA;
	for (int ii=0; ii<2; ++ii) {
	  tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
	  double adjustment = 54 - field;
	  i->_antenna_rel_db += adjustment;
	  erp = i->_power = s->_power * pow(10, i->_antenna_rel_db/10);
	}
	i->_antenna_rel_field = pow(10, i->_antenna_rel_db/20);
      }
      fix_contours(s);
    }else{
      std::cout << "error: " << argv[i] << " unrecognized\n";
    }
  }

  s->write(std::cout);

  return 0;
}
